/******************** (C) COPYRIGHT 2006 STMicroelectronics ********************
* File Name          : usb_mem.c
* Author             : MCD Application Team
* Date First Issued  : 10/27/2003 : V1.0
* Description        : Utility functions for memory transfers to/from PMA
********************************************************************************
* History:
* 06/12/2006 : V2.0  Beta
* 10/27/2003 : V1.0
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "usb_lib.h"
#include "usb_mem.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/



/*******************************************************************************
* Function Name  : UserToPMABufferCopy
* Description    : Copy a buffer from user memory area to packet memory area (PMA)
* Input 1        : pbUsrBuf    = pointer to user memory area
* Input 2        : wPMABufAddr = address into PMA
* Input 3        : wNBytes     = no. of bytes to be copied
* Output         : None
* Return         : None
*******************************************************************************/
void UserToPMABufferCopy(u8 *pbUsrBuf,u16 wPMABufAddr, u16 wNBytes)
{
  u32 n= (wNBytes+1)>>1;   /* n=(wNBytes+1)/2  */
  u32 i,temp1,temp2;
  u16 *pdwVal;
  pdwVal = (u16 *)(wPMABufAddr*2 + PMAAddr);

  for (i = n; i!= 0; i--)
  {
     temp1 = (u16)*pbUsrBuf;
     pbUsrBuf++;
     temp2 = temp1 | (u16)*pbUsrBuf<<8;
     *pdwVal++ = temp2;
     pdwVal++;
     pbUsrBuf++;
  }
} /* UserToPMABufferCopy */



/*******************************************************************************
* Function Name  : PMAToUserBufferCopy
* Description    : Copy a buffer from user memory area to packet memory area (PMA)
* Input 1        : pbUsrBuf    = pointer to user memory area
* Input 2        : wPMABufAddr = address into PMA
* Input 3        : wNBytes     = no. of bytes to be copied
* Output         : pbUsrBuf    = pointer to user memory area with correct data
* Return         : None
*******************************************************************************/
void PMAToUserBufferCopy(u8 *pbUsrBuf,u16 wPMABufAddr, u16 wNBytes)
{
  u32 n= (wNBytes+1)>>1;    /* /2  */
  u32 i;
  u32 *pdwVal;
  pdwVal = (u32 *)(wPMABufAddr*2 + PMAAddr);
  for (i = n; i!= 0; i--)
  {
    *(u16*)pbUsrBuf++ = *pdwVal++;
    pbUsrBuf++;
  }
} /* PMAToUserBufferCopy */

/******************* (C) COPYRIGHT 2006 STMicroelectronics *****END OF FILE****/

